package com.example.application.views.results;

import com.example.application.data.bean.CalculatedMatch;
import com.example.application.data.entity.Player;
import com.example.application.data.service.MatchService;
import com.example.application.data.service.MatchdayService;
import com.example.application.data.service.SeasonService;
import com.example.application.views.abstractnavigation.SeasonAndMatchdayNavigationView;
import com.example.application.views.main.MainView;
import com.vaadin.flow.component.*;
import com.vaadin.flow.component.button.Button;
import com.vaadin.flow.component.dependency.CssImport;
import com.vaadin.flow.component.grid.ColumnTextAlign;
import com.vaadin.flow.component.grid.Grid;
import com.vaadin.flow.component.html.Label;
import com.vaadin.flow.component.icon.Icon;
import com.vaadin.flow.component.icon.VaadinIcon;
import com.vaadin.flow.component.orderedlayout.FlexComponent;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;
import com.vaadin.flow.function.ValueProvider;
import com.vaadin.flow.router.*;
import org.springframework.beans.factory.annotation.Autowired;

import javax.swing.text.NumberFormatter;

@CssImport("./views/results/results-view.css")
@Route(value = "results", layout = MainView.class)
@PageTitle("Results")
public class ResultsView extends SeasonAndMatchdayNavigationView {

    private final MatchService matchService;

    private VerticalLayout matchdayLayout;
    private Label matchdayHeader;
    private Grid<CalculatedMatch> grid;

    private Button prevButton;
    private Button nextButton;

    public ResultsView(@Autowired SeasonService seasonService, @Autowired MatchdayService matchdayService, @Autowired MatchService matchService) {
        super(seasonService, matchdayService);
        this.matchService = matchService;
        addClassName("results-view");
    }

    @Override
    protected String route() {
        return "results";
    }

    @Override
    protected boolean showOnlyMatchdaysWithActivity() {
        return false;
    }

    ////////////
    // LAYOUT //
    ////////////

    private VerticalLayout getMatchdayLayout() {
        if (matchdayLayout == null) {
            matchdayLayout = new VerticalLayout();
        }
        return matchdayLayout;
    }

    private Label getMatchdayHeader() {
        if (matchdayHeader == null) {
            matchdayHeader = new Label();
        }
        return matchdayHeader;
    }

    private Grid<CalculatedMatch> getGrid() {
        if (grid == null) {
            grid = new Grid<>();
        }
        return grid;
    }

    private Button getPrevButton() {
        if (prevButton == null) {
            prevButton = new Button(new Icon(VaadinIcon.ARROW_LEFT));
        }
        return prevButton;
    }

    private Button getNextButton() {
        if (nextButton == null) {
            nextButton = new Button(new Icon(VaadinIcon.ARROW_RIGHT));
        }
        return nextButton;
    }

    @Override
    protected void configureContentLayout() {
        contentLayout = new HorizontalLayout(getPrevButton(), getMatchdayLayout(), getNextButton());
        configureMatchdayLayout();
    }

    private void configureMatchdayLayout() {
        getMatchdayHeader().addClassName("big_header");

        getMatchdayLayout().setPadding(false);
        getMatchdayLayout().add(getMatchdayHeader(), getGrid());
        getMatchdayLayout().setAlignItems(FlexComponent.Alignment.CENTER);
        getMatchdayLayout().addClassName("inner_matchday_layout");

        Label headerPlayer1 = new Label("Player 1");
        headerPlayer1.addClassName("column_header");
        Label headerPlayer2 = new Label("Player 2");
        headerPlayer2.addClassName("column_header");
        Label headerResult = new Label("Result");
        headerResult.addClassName("column_header");

        getGrid().addColumn((ValueProvider<CalculatedMatch, Player>) CalculatedMatch::getPlayer1)
                .setHeader(headerPlayer1)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("13em");
        getGrid().addColumn((ValueProvider<CalculatedMatch, String>) match -> "vs.")
                .setHeader("vs.")
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("3em");
        getGrid().addColumn((ValueProvider<CalculatedMatch, Player>) CalculatedMatch::getPlayer2)
                .setHeader(headerPlayer2)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("13em");
        getGrid().addColumn((ValueProvider<CalculatedMatch, String>) match -> {

            String result = match.getScore1().toString().replace(".0", "")
                    + " : "
                    + match.getScore2().toString().replace(".0", "");
            return result.equals("0 : 0") ? "" : result;
        })
                .setHeader(headerResult)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("6em");

        getGrid().setWidth("36em");
        getGrid().setHeightByRows(true);

        getGrid().addClassName("my_grid");
    }

    /////////////
    // CONTENT //
    /////////////

    @Override
    protected void configureContent() {
        getMatchdayHeader().setText(String.format("Matchday %s", matchdaySelect.getValue().toString()));
        getGrid().setItems(matchService.getCalculatedMatches(matchdaySelect.getValue()));
        configureButtons();
    }

    private void configureButtons() {
        getPrevButton().setEnabled(isMatchDayParamValid(getPrevMatchdayParam()));
        getPrevButton().addClickListener(getButtonClickListener(getPrevMatchdayParam()));

        getNextButton().setEnabled(isMatchDayParamValid(getNextMatchdayParam()));
        getNextButton().addClickListener(getButtonClickListener(getNextMatchdayParam()));
    }

    private ComponentEventListener<ClickEvent<Button>> getButtonClickListener(String matchdayParam) {
        return buttonClickEvent -> getUI().ifPresent(ui -> navigate(ui, seasonParam, matchdayParam));
    }

    private String getPrevMatchdayParam() {
        return String.valueOf(Integer.parseInt(matchdayParam) - 1);
    }

    private String getNextMatchdayParam() {
        return String.valueOf(Integer.parseInt(matchdayParam) + 1);
    }

}
