package com.example.application.views.table;

import com.example.application.data.bean.PlayerForTable;
import com.example.application.data.entity.Player;
import com.example.application.data.service.MatchdayService;
import com.example.application.data.service.PlayerService;
import com.example.application.data.service.SeasonService;
import com.example.application.views.abstractnavigation.SeasonAndMatchdayNavigationView;
import com.example.application.views.main.MainView;
import com.vaadin.flow.component.dependency.CssImport;
import com.vaadin.flow.component.grid.ColumnTextAlign;
import com.vaadin.flow.component.grid.Grid;
import com.vaadin.flow.component.html.Label;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.function.ValueProvider;
import com.vaadin.flow.router.PageTitle;
import com.vaadin.flow.router.Route;
import com.vaadin.flow.router.RouteAlias;
import org.springframework.beans.factory.annotation.Autowired;

@CssImport("./views/table/table-view.css")
@Route(value = "table", layout = MainView.class)
@RouteAlias(value = "", layout = MainView.class)
@PageTitle("Table")
public class TableView extends SeasonAndMatchdayNavigationView {

    private final PlayerService playerService;

    private Grid<PlayerForTable> grid;

    public TableView(@Autowired SeasonService seasonService, @Autowired MatchdayService matchdayService, @Autowired PlayerService playerService) {
        super(seasonService, matchdayService);
        this.playerService = playerService;
        addClassName("table-view");
    }

    @Override
    protected String route() {
        return "table";
    }

    @Override
    protected boolean showOnlyMatchdaysWithActivity() {
        return true;
    }

    ////////////
    // LAYOUT //
    ////////////

    private Grid<PlayerForTable> getGrid() {
        if (grid == null) {
            grid = new Grid<>();
        }
        return grid;
    }

    @Override
    protected void configureContentLayout() {
        contentLayout = new HorizontalLayout(getGrid());

        Label headerPlace = new Label("Place");
        headerPlace.addClassName("column_header");

        Label headerPlayer = new Label("Player");
        headerPlayer.addClassName("column_header");

        Label headerMatchesPlayed = new Label("Played");

        Label headerMatchPoints = new Label("Points");
        headerMatchPoints.addClassName("column_header");

        Label headerWon = new Label("W");

        Label headerDrawn = new Label("D");

        Label headerLost = new Label("L");

        Label headerGames = new Label("Games");

        Label headerDiff = new Label("Diff");
        headerDiff.addClassName("column_header");

        getGrid().addColumn((ValueProvider<PlayerForTable, Integer>) PlayerForTable::getPlace)
                .setHeader(headerPlace)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("5em");

        getGrid().addColumn((ValueProvider<PlayerForTable, Player>) PlayerForTable::getPlayer)
                .setHeader(headerPlayer)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("13em");

        getGrid().addColumn((ValueProvider<PlayerForTable, Integer>) PlayerForTable::getAmountOfMatches)
                .setHeader(headerMatchesPlayed)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("5em");

        getGrid().addColumn((ValueProvider<PlayerForTable, Integer>) PlayerForTable::getMatchPoints)
                .setHeader(headerMatchPoints)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("6em");

        getGrid().addColumn((ValueProvider<PlayerForTable, Integer>) PlayerForTable::getAmountOfMatchesWon)
                .setHeader(headerWon)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("3em");

        getGrid().addColumn((ValueProvider<PlayerForTable, Integer>) PlayerForTable::getAmountOfMatchesDrawn)
                .setHeader(headerDrawn)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("3em");

        getGrid().addColumn((ValueProvider<PlayerForTable, Integer>) PlayerForTable::getAmountOfMatchesLost)
                .setHeader(headerLost)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("3em");

        getGrid().addColumn((ValueProvider<PlayerForTable, String>) playerForTable -> {
            //noinspection CodeBlock2Expr
            return playerForTable.getGamePointsForSelf().toString().replace(".0", "") // TODO: make this reusable
                    + " : "
                    + playerForTable.getGamePointsForOpponents().toString().replace(".0", "");
        })
                .setHeader(headerGames)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("6em");

        getGrid().addColumn((ValueProvider<PlayerForTable, String>) playerForTable -> {
            Double diff = playerForTable.getGamePointDiff();
            String diffString = playerForTable.getGamePointDiff().toString().replace(".0", "");
            return diff > 0 ? "+" + diffString : diffString;
        })
                .setHeader(headerDiff)
                .setTextAlign(ColumnTextAlign.CENTER)
                .setWidth("3em");

        getGrid().setWidth("49em");
        getGrid().setHeightByRows(true);
        getGrid().addClassName("my_grid");
    }

    /////////////
    // CONTENT //
    /////////////

    @Override
    protected void configureContent() {
        getGrid().setItems(playerService.getPlayersForTable(matchdaySelect.getValue()));
    }
}
